#!perl
#
# Forlix SrcDS Query v2.0 - 2010/05/22
# http://forlix.org/, df@forlix.org
#
# Copyright (c) 2008-2010 Dominik Friedrichs

use strict;
use Fcntl qw(LOCK_EX);

#use warnings;
#use CGI::Carp qw(fatalsToBrowser);

die("Not called by SSI\n")
  if($ENV{'SERVER_PROTOCOL'} ne 'INCLUDED');

my %appid =
(
  240, 'CS:S',
  440, 'TF2',
  0, '-',
);

my %srv_addr =
(
  'fxpure_cstrike1',  '178.77.71.84:27101',
  'fxpure_cstrike2',  '178.77.71.84:27102',
  'fxpure_tf1',       '178.77.71.84:27201',
  'fxpure_tf2',       '178.77.71.84:27202',
);

my $cachetime = 150;

sub readcache(\$\%\%);
sub writecache(\$\%\%);
sub f_time($);
sub printout(\$\$\%\%);

sub a2s_query(\$\$\%\%);


my $time_c = time();
my $srv_id = $ENV{'QUERY_STRING'};

die("Unknown server\n")
  if(!defined($srv_addr{$srv_id}));

my ($ip, $port) = split(':', $srv_addr{$srv_id});

my $cachefile = $srv_id.'.txt';
my $lockfile = $srv_id.'.lck';

open(LOCK, '>', $lockfile) || die($!);
flock(LOCK, LOCK_EX);

my (%a2sinfo, %a2splayer);
readcache($cachefile, %a2sinfo, %a2splayer);

if(!defined($a2sinfo{'qtime'})
|| $time_c-$a2sinfo{'qtime'} > $cachetime)
{
  require('srcds_qry.pm');

  a2s_query($ip, $port, %a2sinfo, %a2splayer);
  $a2sinfo{'qtime'} = $time_c;
  writecache($cachefile, %a2sinfo, %a2splayer);
}

close(LOCK);
unlink($lockfile);

if(defined($a2sinfo{'error'}))
{
  $a2sinfo{'name'} = 'Query failed';
  $a2sinfo{'map'} = '-';
  $a2sinfo{'maxplayers'} = 0;
  $a2sinfo{'numplayers'} = 0;
  $a2sinfo{'appid'} = 0;

  $a2splayer{'sets'} = 1;
  $a2splayer{'0'} = $a2sinfo{'error'}.',&nbsp;,&nbsp;';
}
elsif(!$a2splayer{'sets'})
{
  $a2splayer{'sets'} = 1;
  $a2splayer{'0'} = 'No players on server,&nbsp;,&nbsp;';
}

printout($ip, $port, %a2sinfo, %a2splayer);


sub f_time($)
{
  my ($time_s) = @_;
  my $time_f;

  my $hrs = int($time_s/3600);
  my $mins = ($time_s/60)%60;
  my $secs = $time_s%60;

  if($time_s < 60)
  {
    $time_f = $secs.'s';
  }
  elsif($time_s < 3600)
  {
    $time_f = $mins.'m '.$secs.'s';
  }
  else
  {
    $time_f = $hrs.'h '.$mins.'m';
  }

  return($time_f);
}


sub readcache(\$\%\%)
{
  my ($file, $a2sinfo, $a2splayer) = @_;
  %$a2sinfo = ();
  %$a2splayer = ();
  open(CACHE, '<', $$file) || return(0);

  while(<CACHE>)
  {
    chomp($_);
    my ($type, $data) = split('/', $_);
    my ($key, $value) = split('=', $data);

    $$a2sinfo{$key} = $value
      if($type == 1);

    $$a2splayer{$key} = $value
      if($type == 2);
  }

  close(CACHE);
  return(1);
}


sub writecache(\$\%\%)
{
  my ($file, $a2sinfo, $a2splayer) = @_;
  my @cache;

  for my $key (keys %$a2sinfo)
  {
    push(@cache, '1/'.$key.'='.$$a2sinfo{$key});
  }
  for my $key (keys %$a2splayer)
  {
    push(@cache, '2/'.$key.'='.$$a2splayer{$key});
  }

  @cache = sort(@cache);
  $" = "\n";
  open(CACHE, '>', $$file) || return(0);
  print(CACHE "@cache");
  close(CACHE);

  return(1);
}


sub printout(\$\$\%\%)
{
  my ($str_ip, $str_port, $a2sinfo, $a2splayer) = @_;
  my $cache_age = $time_c-$$a2sinfo{'qtime'};

  print("Content-Type: text/html\n\n");

print(<<'EOT');
<table class="srcds_table">
<colgroup>
<col class="player" />
<col class="score" />
<col class="time" />
</colgroup>
<tr>
EOT

  print('<th colspan="3" class="hostname">'.$$a2sinfo{'name'}."</th>\n");

  print("</tr>\n");
  print("<tr>\n");

  print('<th colspan="1" class="map">Map: <span class="normal">'.$$a2sinfo{'map'}."</span></th>\n");
  print('<th colspan="2" class="numplayers">Players: <span class="normal">'
        .$$a2sinfo{'numplayers'}.'/'.$$a2sinfo{'maxplayers'}."</span></th>\n");

print(<<'EOT');
</tr>
<tr>
<th>Player</th>
<th>Score</th>
<th>Time</th>
</tr>
EOT

  for(my $i = 0; $i < $$a2splayer{'sets'}; $i++)
  {
    my ($player, $score, $time) = split(',', $$a2splayer{$i});
    $time = ($time == -1)? 'BOT' : f_time($time+$cache_age)
      if($time ne '&nbsp;');

    print("<tr>\n");
    print('<td class="player">'.$player."</td>\n");
    print('<td class="score">'.$score."</td>\n");
    print('<td class="time">'.$time."</td>\n");
    print("</tr>\n");
  }

  print("<tr>\n");
  print('<th colspan="1" class="footlt">Cache: <span class="normal">'.($cache_age? f_time($cache_age) : 'fresh')."</span></th>\n");
  print('<th colspan="2" class="footrt">Game: <span class="normal">'.$appid{$$a2sinfo{'appid'}}."</span></th>\n");

  print("</tr>\n");
  print("<tr>\n");

  print('<th colspan="1" class="footlb">IP/Port: <span class="normal">'.$$str_ip.':'.$$str_port."</span></th>\n");
  print('<th colspan="2" class="footrb"><a href="steam://connect/'.$$str_ip.':'.$$str_port.'/">Join Server'."</a></th>\n");
  print("</tr>\n");
  print("</table>\n");

  return(1);
}
